// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/notebooks/v2/service.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NOTEBOOKS_V2_NOTEBOOK_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NOTEBOOKS_V2_NOTEBOOK_CONNECTION_H

#include "google/cloud/notebooks/v2/internal/notebook_retry_traits.h"
#include "google/cloud/notebooks/v2/notebook_connection_idempotency_policy.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/future.h"
#include "google/cloud/internal/retry_policy_impl.h"
#include "google/cloud/no_await_tag.h"
#include "google/cloud/options.h"
#include "google/cloud/polling_policy.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/notebooks/v2/service.pb.h>
#include <google/longrunning/operations.grpc.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace notebooks_v2 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

/// The retry policy for `NotebookServiceConnection`.
class NotebookServiceRetryPolicy : public ::google::cloud::RetryPolicy {
 public:
  /// Creates a new instance of the policy, reset to the initial state.
  virtual std::unique_ptr<NotebookServiceRetryPolicy> clone() const = 0;
};

/**
 * A retry policy for `NotebookServiceConnection` based on counting errors.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - More than a prescribed number of transient failures is detected.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class NotebookServiceLimitedErrorCountRetryPolicy
    : public NotebookServiceRetryPolicy {
 public:
  /**
   * Create an instance that tolerates up to @p maximum_failures transient
   * errors.
   *
   * @note Disable the retry loop by providing an instance of this policy with
   *     @p maximum_failures == 0.
   */
  explicit NotebookServiceLimitedErrorCountRetryPolicy(int maximum_failures)
      : impl_(maximum_failures) {}

  NotebookServiceLimitedErrorCountRetryPolicy(
      NotebookServiceLimitedErrorCountRetryPolicy&& rhs) noexcept
      : NotebookServiceLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}
  NotebookServiceLimitedErrorCountRetryPolicy(
      NotebookServiceLimitedErrorCountRetryPolicy const& rhs) noexcept
      : NotebookServiceLimitedErrorCountRetryPolicy(rhs.maximum_failures()) {}

  int maximum_failures() const { return impl_.maximum_failures(); }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<NotebookServiceRetryPolicy> clone() const override {
    return std::make_unique<NotebookServiceLimitedErrorCountRetryPolicy>(
        maximum_failures());
  }

  // This is provided only for backwards compatibility.
  using BaseType = NotebookServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedErrorCountRetryPolicy<
      notebooks_v2_internal::NotebookServiceRetryTraits>
      impl_;
};

/**
 * A retry policy for `NotebookServiceConnection` based on elapsed time.
 *
 * This policy stops retrying if:
 * - An RPC returns a non-transient error.
 * - The elapsed time in the retry loop exceeds a prescribed duration.
 *
 * In this class the following status codes are treated as transient errors:
 * - [`kUnavailable`](@ref google::cloud::StatusCode)
 */
class NotebookServiceLimitedTimeRetryPolicy
    : public NotebookServiceRetryPolicy {
 public:
  /**
   * Constructor given a `std::chrono::duration<>` object.
   *
   * @tparam DurationRep a placeholder to match the `Rep` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the underlying
   *     arithmetic type used to store the number of ticks. For our purposes it
   *     is simply a formal parameter.
   * @tparam DurationPeriod a placeholder to match the `Period` tparam for @p
   *     duration's type. The semantics of this template parameter are
   *     documented in `std::chrono::duration<>`. In brief, the length of the
   *     tick in seconds, expressed as a `std::ratio<>`. For our purposes it is
   *     simply a formal parameter.
   * @param maximum_duration the maximum time allowed before the policy expires.
   *     While the application can express this time in any units they desire,
   *     the class truncates to milliseconds.
   *
   * @see https://en.cppreference.com/w/cpp/chrono/duration for more information
   *     about `std::chrono::duration`.
   */
  template <typename DurationRep, typename DurationPeriod>
  explicit NotebookServiceLimitedTimeRetryPolicy(
      std::chrono::duration<DurationRep, DurationPeriod> maximum_duration)
      : impl_(maximum_duration) {}

  NotebookServiceLimitedTimeRetryPolicy(
      NotebookServiceLimitedTimeRetryPolicy&& rhs) noexcept
      : NotebookServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}
  NotebookServiceLimitedTimeRetryPolicy(
      NotebookServiceLimitedTimeRetryPolicy const& rhs) noexcept
      : NotebookServiceLimitedTimeRetryPolicy(rhs.maximum_duration()) {}

  std::chrono::milliseconds maximum_duration() const {
    return impl_.maximum_duration();
  }

  bool OnFailure(Status const& status) override {
    return impl_.OnFailure(status);
  }
  bool IsExhausted() const override { return impl_.IsExhausted(); }
  bool IsPermanentFailure(Status const& status) const override {
    return impl_.IsPermanentFailure(status);
  }
  std::unique_ptr<NotebookServiceRetryPolicy> clone() const override {
    return std::make_unique<NotebookServiceLimitedTimeRetryPolicy>(
        maximum_duration());
  }

  // This is provided only for backwards compatibility.
  using BaseType = NotebookServiceRetryPolicy;

 private:
  google::cloud::internal::LimitedTimeRetryPolicy<
      notebooks_v2_internal::NotebookServiceRetryTraits>
      impl_;
};

/**
 * The `NotebookServiceConnection` object for `NotebookServiceClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `NotebookServiceClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `NotebookServiceClient`.
 *
 * To create a concrete instance, see `MakeNotebookServiceConnection()`.
 *
 * For mocking, see `notebooks_v2_mocks::MockNotebookServiceConnection`.
 */
class NotebookServiceConnection {
 public:
  virtual ~NotebookServiceConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StreamRange<google::cloud::notebooks::v2::Instance> ListInstances(
      google::cloud::notebooks::v2::ListInstancesRequest request);

  virtual StatusOr<google::cloud::notebooks::v2::Instance> GetInstance(
      google::cloud::notebooks::v2::GetInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  CreateInstance(
      google::cloud::notebooks::v2::CreateInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> CreateInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::CreateInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  CreateInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  UpdateInstance(
      google::cloud::notebooks::v2::UpdateInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> UpdateInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::UpdateInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  UpdateInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::OperationMetadata>>
  DeleteInstance(
      google::cloud::notebooks::v2::DeleteInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> DeleteInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::DeleteInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::OperationMetadata>>
  DeleteInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  StartInstance(
      google::cloud::notebooks::v2::StartInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> StartInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::StartInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  StartInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>> StopInstance(
      google::cloud::notebooks::v2::StopInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> StopInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::StopInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>> StopInstance(
      google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  ResetInstance(
      google::cloud::notebooks::v2::ResetInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> ResetInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::ResetInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  ResetInstance(google::longrunning::Operation const& operation);

  virtual StatusOr<
      google::cloud::notebooks::v2::CheckInstanceUpgradabilityResponse>
  CheckInstanceUpgradability(
      google::cloud::notebooks::v2::CheckInstanceUpgradabilityRequest const&
          request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  UpgradeInstance(
      google::cloud::notebooks::v2::UpgradeInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> UpgradeInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::UpgradeInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  UpgradeInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  RollbackInstance(
      google::cloud::notebooks::v2::RollbackInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> RollbackInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::RollbackInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  RollbackInstance(google::longrunning::Operation const& operation);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  DiagnoseInstance(
      google::cloud::notebooks::v2::DiagnoseInstanceRequest const& request);

  virtual StatusOr<google::longrunning::Operation> DiagnoseInstance(
      NoAwaitTag,
      google::cloud::notebooks::v2::DiagnoseInstanceRequest const& request);

  virtual future<StatusOr<google::cloud::notebooks::v2::Instance>>
  DiagnoseInstance(google::longrunning::Operation const& operation);

  virtual StreamRange<google::cloud::location::Location> ListLocations(
      google::cloud::location::ListLocationsRequest request);

  virtual StatusOr<google::cloud::location::Location> GetLocation(
      google::cloud::location::GetLocationRequest const& request);

  virtual StatusOr<google::iam::v1::Policy> SetIamPolicy(
      google::iam::v1::SetIamPolicyRequest const& request);

  virtual StatusOr<google::iam::v1::Policy> GetIamPolicy(
      google::iam::v1::GetIamPolicyRequest const& request);

  virtual StatusOr<google::iam::v1::TestIamPermissionsResponse>
  TestIamPermissions(google::iam::v1::TestIamPermissionsRequest const& request);

  virtual StreamRange<google::longrunning::Operation> ListOperations(
      google::longrunning::ListOperationsRequest request);

  virtual StatusOr<google::longrunning::Operation> GetOperation(
      google::longrunning::GetOperationRequest const& request);

  virtual Status DeleteOperation(
      google::longrunning::DeleteOperationRequest const& request);

  virtual Status CancelOperation(
      google::longrunning::CancelOperationRequest const& request);
};

/**
 * A factory function to construct an object of type
 * `NotebookServiceConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of NotebookServiceClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `NotebookServiceConnection`. Expected options are any of the types
 * in the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::notebooks_v2::NotebookServicePolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `NotebookServiceConnection` created
 * by this function.
 */
std::shared_ptr<NotebookServiceConnection> MakeNotebookServiceConnection(
    Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace notebooks_v2
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_NOTEBOOKS_V2_NOTEBOOK_CONNECTION_H
